# Observing Analytics

A listener can be registered to a Stream to allow for the host application to see all of the analytics that are being tracked by the Appcues Flutter Plugin.

## Registering to the Analytics Event Stream

You can register to receive Appcues analytics events using the `onAnalyticEvent` [Stream](https://api.flutter.dev/flutter/dart-async/Stream-class.html):

```dart
// Add the listener
var listener = Appcues.onAnalyticEvent.listen((event) {
    // analytic: Enum string indicating the type of the analytic. Possible values listed below.
    // value: String containing the primary value of the analytic being tracked. For events - the event name, for screens - the screen title, for identify - the user ID, for group - the group ID.
    // properties: Object containing the properties that provide additional context about the analytic.
    // isInternal: Boolean indicating if the analytic was internally generated by the SDK, as opposed to passed in from the host application.
    print("${event.analytic}: value=${event.value}, isInternal=${event.isInternal}, properties=${event.properties}");
});
...

//Removes the listener
listener.cancel();
 ```

## Using the Analytics Tracking Data

There are four different possible values for `analytic` and two possible values for `isInternal`:

| `analytic`     | `isInternal` | SDK call              |
| -------------- | ------------ | --------------------- |
| `EVENT`        | `true`       | N/A                   |
| `EVENT`        | `false`      | `Appcues.track()`     |
| `SCREEN`       | `false`      | `Appcues.screen()`    |
| `IDENTIFY`     | `false`      | `Appcues.identify()`  |
| `GROUP`        | `false`      | `Appcues.group()`     |

Events with `isInternal == true` correspond to internal SDK events - these capture anything that is generated automatically inside of the Appcues SDK, including flow events and session events.

All other analytics - these are all the screens, events, user or group identities that are passed into the SDK from the host application have `isInternal == false`.
